@@ -1,14 +1,16 @@
 package main
 
 import (
+	"errors"
 	"flag"
 	"fmt"
 	"log"
 	"os"
 	"path/filepath"
 	"strings"
+	"syscall"
 
-	"github.com/docker/distribution/reference"
+	"github.com/distribution/reference"
 	dockerregistrytypes "github.com/docker/docker/api/types/registry"
 	docker "github.com/fsouza/go-dockerclient"
 	"k8s.io/klog"
@@ -32,7 +34,7 @@ func main() {
 	var version bool
 	var mountSpecs stringSliceFlag
 
-	VERSION := "1.2.11"
+	VERSION := "1.2.15"
 	arguments := stringMapFlag{}
 
 	flag.Var(&tags, "t", "The name to assign this image, if any. May be specified multiple times.")
@@ -90,10 +92,14 @@ func main() {
 	options.Out, options.ErrOut = os.Stdout, os.Stderr
 	authConfigurations, err := docker.NewAuthConfigurationsFromDockerCfg()
 	if err != nil {
-		log.Fatalf("reading authentication configurations: %v", err)
+		if errors.Is(err, syscall.ENOENT) {
+			klog.Warning("No docker configuration found")
+		} else {
+			log.Fatalf("reading authentication configurations: %v", err)
+		}
 	}
 	if authConfigurations == nil {
-		klog.V(4).Infof("No authentication secrets found")
+		klog.V(4).Info("No authentication secrets found")
 	}
 
 	options.AuthFn = func(name string) ([]dockerregistrytypes.AuthConfig, bool) {
